#include "TFT_objects.h"
#include "TFT_resources.h"
#include "built_in.h"


// TFT module connections
sbit TFT_BLED at GPIOE_ODR.B9;
sbit TFT_CS at GPIOE_ODR.B15;
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_WR at GPIOE_ODR.B11;
// End TFT module connections

// Touch Panel module connections
sbit DriveX_Left at GPIOB_ODR.B1;
sbit DriveX_Right at GPIOB_ODR.B8;
sbit DriveY_Up at GPIOB_ODR.B9;
sbit DriveY_Down at GPIOB_ODR.B0;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 1500;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TCircleButton *local_circle_button;
TCircleButton *exec_circle_button;
int circle_button_order;


void Init_ADC() {
  ADC_Set_Input_Channel(_ADC_CHANNEL_8 | _ADC_CHANNEL_9);
  ADC1_Init();
  ADC_CCRbits.ADCPRE = 3; // PCLK2/8
  Delay_ms(100);
}
static void InitializeTouchPanel() {
  Init_ADC();
  TFT_Init_ILI9341_8bit(320, 240);

  TP_TFT_Init(320, 240, 8, 9);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Screen1;
  TCircleButton          CircleButton1;
char CircleButton1_Caption[8] = "Push Me";

  TCircleButton          * const code Screen1_CircleButtons[1]=
         {
         &CircleButton1        
         };



static void InitializeObjects() {
  Screen1.Color                     = 0xF81F;
  Screen1.Width                     = 320;
  Screen1.Height                    = 240;
  Screen1.CircleButtonsCount        = 1;
  Screen1.CircleButtons             = Screen1_CircleButtons;
  Screen1.ObjectsCount              = 1;


  CircleButton1.OwnerScreen     = &Screen1;
  CircleButton1.Order           = 0;
  CircleButton1.Left            = 82;
  CircleButton1.Top             = 45;
  CircleButton1.Radius          = 72;
  CircleButton1.Pen_Width       = 1;
  CircleButton1.Pen_Color       = 0x0000;
  CircleButton1.Visible         = 1;
  CircleButton1.Active          = 1;
  CircleButton1.Transparent     = 1;
  CircleButton1.Caption         = CircleButton1_Caption;
  CircleButton1.TextAlign       = _taCenter;
  CircleButton1.TextAlignVertical= _tavMiddle;
  CircleButton1.FontName        = Tahoma19x23_Regular;
  CircleButton1.PressColEnabled = 1;
  CircleButton1.Font_Color      = 0x0000;
  CircleButton1.VerticalText    = 0;
  CircleButton1.Gradient        = 1;
  CircleButton1.Gradient_Orientation = 0;
  CircleButton1.Gradient_Start_Color = 0x9FFF;
  CircleButton1.Gradient_End_Color = 0xFFF3;
  CircleButton1.Color           = 0xC618;
  CircleButton1.Press_Color     = 0xE71C;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetCircleButton(index)        CurrentScreen->CircleButtons[index]


void DrawCircleButton(TCircleButton *ACircle_button) {
unsigned int ALeft, ATop;
  if (ACircle_button->Visible != 0) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(ACircle_button->Transparent, ACircle_button->Press_Color, ACircle_button->Gradient, ACircle_button->Gradient_Orientation,
                    ACircle_button->Gradient_End_Color, ACircle_button->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(ACircle_button->Transparent, ACircle_button->Color, ACircle_button->Gradient, ACircle_button->Gradient_Orientation,
                    ACircle_button->Gradient_Start_Color, ACircle_button->Gradient_End_Color);
    }
    TFT_Set_Pen(ACircle_button->Pen_Color, ACircle_button->Pen_Width);
    TFT_Circle(ACircle_button->Left + ACircle_button->Radius,
               ACircle_button->Top + ACircle_button->Radius,
               ACircle_button->Radius);
    if (ACircle_button->VerticalText != 0)
      TFT_Set_Font(ACircle_button->FontName, ACircle_button->Font_Color, FO_VERTICAL_COLUMN);
    else
      TFT_Set_Font(ACircle_button->FontName, ACircle_button->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(ACircle_button->Caption, ACircle_button->Left, ACircle_button->Top);

    if (ACircle_button->TextAlign == _taLeft)
      ALeft = ACircle_button->Left + 4;
    else if (ACircle_button->TextAlign == _taCenter)
      ALeft = ACircle_button->Left + (ACircle_button->Radius*2 - caption_length) / 2;
    else if (ACircle_button->TextAlign == _taRight)
      ALeft = ACircle_button->Left + ACircle_button->Radius*2 - caption_length - 4;

    if (ACircle_button->TextAlignVertical == _tavTop)
      ATop = ACircle_button->Top + 3;
    else if (ACircle_button->TextAlignVertical == _tavMiddle)
      ATop = ACircle_button->Top + ACircle_button->Radius - caption_height / 2;
    else if (ACircle_button->TextAlignVertical == _tavBottom)
      ATop = ACircle_button->Top + (ACircle_button->Radius * 2) - caption_height - 4;

    TFT_Write_Text(ACircle_button->Caption, ALeft, ATop);
  }
}

void DrawScreen(TScreen *aScreen) {
 unsigned int order;
  unsigned short circle_button_idx;
  TCircleButton *local_circle_button;
  char save_bled;

  object_pressed = 0;
  order = 0;
  circle_button_idx = 0;
  CurrentScreen = aScreen;

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    TFT_BLED           = 0;
    TFT_Init_ILI9341_8bit(CurrentScreen->Width, CurrentScreen->Height);
    TP_TFT_Init(CurrentScreen->Width, CurrentScreen->Height, 8, 9);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(CurrentScreen->Color);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);


  while (order < CurrentScreen->ObjectsCount) {
    if (circle_button_idx < CurrentScreen->CircleButtonsCount) {
      local_circle_button = GetCircleButton(circle_button_idx);
      if (order == local_circle_button->Order) {
        circle_button_idx++;
        order++;
        DrawCircleButton(local_circle_button);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  circle_button_order = -1;
  //  Circle Buttons
  for ( _object_count = 0 ; _object_count < CurrentScreen->CircleButtonsCount ; _object_count++ ) {
    local_circle_button = GetCircleButton(_object_count);
    if (local_circle_button->Active != 0) {
      if (IsInsideObject(X, Y, local_circle_button->Left, local_circle_button->Top,
                        (local_circle_button->Radius * 2), (local_circle_button->Radius * 2)) == 1) {
        circle_button_order = local_circle_button->Order;
        exec_circle_button = local_circle_button;
      }
    }
  }

  _object_count = -1;
  if (circle_button_order >  _object_count )
    _object_count = circle_button_order;
}


void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_circle_button  = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == circle_button_order) {
      if (exec_circle_button->Active != 0) {
        if (exec_circle_button->OnPressPtr != 0) {
          exec_circle_button->OnPressPtr();
          return;
        }
      }
    }

  }
}

void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // Circle Button
    case 5: {
      if (PressedObject != 0) {
        exec_circle_button = (TCircleButton*)PressedObject;
        if ((exec_circle_button->PressColEnabled != 0) && (exec_circle_button->OwnerScreen == CurrentScreen)) {
          DrawCircleButton(exec_circle_button);
        }
        break;
      }
      break;
    }
  }


  Get_Object(X, Y);


  if (_object_count != -1) {
  // Circle Buttons
    if (_object_count == circle_button_order) {
      if (exec_circle_button->Active != 0) {
        if (exec_circle_button->OnUpPtr != 0)
          exec_circle_button->OnUpPtr();
        if (PressedObject == (void *)exec_circle_button)
          if (exec_circle_button->OnClickPtr != 0)
            exec_circle_button->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_circle_button  = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == circle_button_order) {
      if (exec_circle_button->Active != 0) {
        if (exec_circle_button->PressColEnabled != 0) {
          object_pressed = 1;
          DrawCircleButton(exec_circle_button);
        }
        PressedObject = (void *)exec_circle_button;
        PressedObjectType = 5;
        if (exec_circle_button->OnDownPtr != 0) {
          exec_circle_button->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU() {
  GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_9);
  TFT_BLED = 1;
  TFT_Set_Default_Mode();
  TP_TFT_Set_Default_Mode();
}

void Start_TP() {
  Init_MCU();

  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(239, 3700, 224, 3679);    // Set calibration constants

  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}