#ifndef FREERTOS_CONFIG_H
#define FREERTOS_CONFIG_H

//-----------------------------------------------------------
// Application specific definitions.
// These definitions should be adjusted for your particular hardware and
// application requirements.
//-----------------------------------------------------------

#define configUSE_PREEMPTION            1
#define configUSE_IDLE_HOOK             0
#define configUSE_TICK_HOOK             0

#define configCPU_CLOCK_HZ              Clock_kHz() * 1000
#define configTICK_RATE_HZ              ((TickType_t)1000)
#define configSYSTICK_USE_CORE_CLOCK    1
#define configSYSTICK_CLOCK_DIVIDER     1
#define configSYSTICK_CLOCK_HZ                          \
    ((configCPU_CLOCK_HZ)/configSYSTICK_CLOCK_DIVIDER)

#define configMAX_PRIORITIES            (16)
#define configMINIMAL_STACK_SIZE        ((uint16_t)128)
#define configTOTAL_HEAP_SIZE           16384    //8192
#define configMAX_TASK_NAME_LEN         (32)
#define configUSE_TRACE_FACILITY        0
#define configUSE_16_BIT_TICKS          0
#define configIDLE_SHOULD_YIELD         1

// Added to enable timers VG July 12, 2018
#define configUSE_TIMERS                  1
#define configTIMER_TASK_PRIORITY         10     // Perhaps 3 may be better
#define configTIMER_QUEUE_LENGTH          5      // Perhaps 10 may be better
#define configTIMER_TASK_STACK_DEPTH      5      // configMINIMAL_STACK_SIZE
#define configSUPPORT_DYNAMIC_ALLOCATION  1

// Added to enable notifications VG July 15, 2018
#define configUSE_TASK_NOTIFICATIONS      1

// Co-routine definitions.
#define configUSE_CO_ROUTINES           0
#define configMAX_CO_ROUTINE_PRIORITIES (2)

#define configUSE_MUTEXES                    1
#define configUSE_COUNTING_SEMAPHORES        1
#define configUSE_ALTERNATIVE_API            0
#define configCHECK_FOR_STACK_OVERFLOW       0
#define configUSE_RECURSIVE_MUTEXES          1
#define configQUEUE_REGISTRY_SIZE            10  // 10 may be better
#define configGENERATE_RUN_TIME_STATS        0

// Set the following definitions to 1 to include the API function,
// or zero to exclude the API function.

#define INCLUDE_vTaskPrioritySet             1
#define INCLUDE_uxTaskPriorityGet            1
#define INCLUDE_vTaskDelete                  1
#define INCLUDE_vTaskCleanUpResources        0  //??????
#define INCLUDE_vTaskSuspend                 1
#define INCLUDE_vTaskDelayUntil              1
#define INCLUDE_vTaskDelay                   1

// Added 2018/08/01 VG
#define INCLUDE_xResumeFromISR                  1
#define INCLUDE_xTaskGetSchedulerState          1
#define INCLUDE_xTaskGetCurrentTaskHandle       1
#define INCLUDE_uxTaskGetStackHighWaterMark     0
#define INCLUDE_xTaskGetIdleTaskHandle          0
#define INCLUDE_eTaskGetState                   0
#define INCLUDE_xEventGroupSetBitFromISR        1
#define INCLUDE_xTimerPendFunctionCall          0
#define INCLUDE_xTaskAbortDelay                 0
#define INCLUDE_xTaskGetHandle                  0
#define INCLUDE_xTaskResumeFromISR              1



// This demo makes use of one or more example stats formatting functions.
// These format the raw data provided by the uxTaskGetSystemState() function
// in to human readable ASCII form.  See the notes in the implementation
// of vTaskList() within FreeRTOS/Source/tasks.c for limitations.
#define configUSE_STATS_FORMATTING_FUNCTIONS        1

// This is the raw value as per the Cortex-M3 NVIC.
// Values can be 255 (lowest) to 0 (1?) (highest).
#define configKERNEL_INTERRUPT_PRIORITY             0xFF
// !!!! configMAX_SYSCALL_INTERRUPT_PRIORITY must not be set to zero !!!!
// See http://www.FreeRTOS.org/RTOS-Cortex-M3-M4.html.
// Equivalent to 0xb0, or priority 11.
#define configMAX_SYSCALL_INTERRUPT_PRIORITY        0x10

// Normal assert() mechanics without relying on assert.h header file.
#define configASSERT(x) if ((x) == 0) { taskDISABLE_INTERRUPTS(); while (1); }
//#define configASSERT(x) if ((x) == 0) {}


#endif // FREERTOS_CONFIG_H